import com.ibm.websphere.rainmaker.logging.Output;
import com.ibm.ws.xs.app.DomainConnectionUpdater;


def createDomain(domain, owner = false) {
    if (getDomains([:], false).size() == 0) {
        storage.current.domain = domain.name
    }
    DomainConnectionUpdater.getInstance().setConnectionInfo(DomainConnectionUpdater.CURRENT_DOMAIN);
    return createResource('domains', domain, owner)
}

def deleteDomain(id) {
    def dom = getDomain(id)

    if (dom != null) {
        if (dom.name.equals(storage.current.domain[])) {
            storage.current.domain = ""
        }
    }
    def rv = deleteResource('domains', id)
    def refs = getResources('domainsservers', [domainid: id], false)

    refs.each {
        deleteResource('domainsservers', it.id)
    }

    refs = getResources('domainskeystores', [domainid: id], false)

    refs.each {
        deleteResource('domainskeystores', it.id)
    }

    return rv
}

def getDomain(id) {
    //invokeMethod("groovy/rainmaker/keystores/Keystores.groovy", "refreshKeystores", null);
    def rv = getResource('domains', [id: id], ['id', 'name', 'jmxuser', 'jmxpass',  'customcredgenerator', 'credgeneratorproperties', 'serverprops'])
    
    return rv
}

def getDomains(filters, owner = false) {
    def _filters = filters.clone()
    def rv = getResources( 'domains', _filters, false, ['id', 'name', 'jmxuser', 'jmxpass', 'customcredgenerator', 'credgeneratorproperties', 'serverprops'])

    return rv
}

def updateDomain(domain) {
    // Map verify password to normal password update as expected by updateResource.
    if (domain.verification != null) {
       domain.put("keystorepassword", domain.value);
       domain.remove("value");
       domain.remove("verification");
       domain.remove("widget");
    }
    def rv = updateResource('domains', domain, ['id', 'name', 'jmxuser', 'jmxpass', 'customcredgenerator', 'credgeneratorproperties', 'serverprops'])
    DomainConnectionUpdater.getInstance().setConnectionInfo(DomainConnectionUpdater.CURRENT_DOMAIN);
    
    return rv
}

def sanitizeCatserverRec(rv) {
     // Create a version that's safe to write out to the logs.
     def sanitized = rv.clone().collect() { it.clone() }
     sanitized.each { srv ->
         if (srv.security.jmxPass != null) {
             srv.security = srv.security.clone()
             srv.security.jmxPass = "******"
         }
         if (srv.jmxPort != null)
        	 srv.jmxPort = ""
     }
     return sanitized
 }

def convertDomainToCatserverRec(filters) {
    def hits = getDomains(filters, false)

    if (hits.size() == 0) {
        return null
    }

    def domain  = hits[0]
    def servers = getDomainsservers([domainid: domain.id], false)
    def rv      = []

    servers.each { srv ->
        def server = getResource("servers", [id: srv.serverid], ["alias", "name", "listenerport", "jmxport"])

        if (server != null) {
            def x = [ 
                serverName: server.alias?.trim(), 
                hostName: server.name?.trim(), 
                orbPort: server.listenerport.toString(),
                //jmxPort: server.jmxport.toLong(),
                security: [ 
                    customCredGenerator: domain.customcredgenerator?.trim(),
                    credGeneratorProperties: domain.credgeneratorproperties?.trim(),
                    serverProps: domain.serverprops?.trim() ] ]

            rv << x
        } else {
          Output.error("Bad server id ${srv.serverid}, ignoring")
        }
     }

     
     Output.info("Conversion: ${sanitizeCatserverRec(rv)}")
     return rv
}
